from typing import TypedDict

Map = TypedDict(
    'Map',
    {
        'Name': str,
        'Map Enabled': bool,
        'Map Locked': bool,
        'Map Internal Name': str,
        'Avg. Player Level': float,
        'Raid Timer': int,
        'Max Players': float,
        'Min Players': float,
        'Required Player Level': float,
    },
    total=False,
)



from __future__ import annotations

import aiohttp

from typing import Dict, Optional, List
from asyncio import get_event_loop, AbstractEventLoop

from .ammunition import Ammunition
from .armor import Armor
from .backpack import Backpack
from .barter import Barter
from .food import Food
from .grenade import Grenade
from .headphone import Headphone
from .item import Item
from .key import Key
from .maps import Map
from .requester import HTTPRequester

__all__ = ('TVCClient',)


class TVCClient:

    def __init__(
        self,
        token: str,
        loop: Optional[AbstractEventLoop] = None,
        session: aiohttp.ClientSession = aiohttp.ClientSession(),
    ) -> None:
        self.loop: AbstractEventLoop = get_event_loop() if loop is None else loop
        self.__requester: HTTPRequester = HTTPRequester(token=token, session=session, loop=loop)
        self.token: str = token

        self._clear()

    def _clear(self) -> None:
        self._ammunition: Dict[str, Ammunition] = {}
        self._armors: Dict[str, Armor] = {}
        self._backpacks: Dict[str, Backpack] = {}
        self._barters: Dict[str, Barter] = {}
        self._foods: Dict[str, Food] = {}
        self._grenades: Dict[str, Grenade] = {}
        self._items: Dict[str, Item] = {}
        self._keys: Dict[str, Key] = {}

    def start(self) -> None:
        self.loop.run_until_complete(self.load_endpoints())

    def get_ammunition(self, name: str) -> Optional[Ammunition]:
        return self._ammunition.get(name)

    def get_armor(self, name: str) -> Optional[Armor]:
        return self._armors.get(name)

    def get_backpack(self, name: str) -> Optional[Backpack]:
        return self._backpacks.get(name)

    def get_barter(self, name: str) -> Optional[Barter]:
        return self._barters.get(name)

    def get_food(self, name: str) -> Optional[Food]:
        return self._foods.get(name)

    def get_grenade(self, name: str) -> Optional[Grenade]:
        return self._grenades.get(name)

    def get_item(self, name: str) -> Optional[Item]:
        return self._items.get(name)

    def get_key(self, name: str) -> Optional[Key]:
        return self._keys.get(name)

    async def load_endpoints(self) -> None:
        self._clear()

        armors = await self.fetch_armor()
        ammunition = await self.fetch_ammunition()
        backpacks = await self.fetch_backpack()
        barters = await self.fetch_barter()
        foods = await self.fetch_food()
        grenades = await self.fetch_grenade()
        items = await self.fetch_item()
        keys = await self.fetch_key()

        for armor in armors:
            self._armors[armor.name] = armor

        for ammo in ammunition:
            self._ammunition[ammo.name] = ammo

        for backpack in backpacks:
            self._backpacks[backpack.name] = backpack

        for barter in barters:
            self._barters[barter.name] = barter

        for food in foods:
            self._foods[food.name] = food

        for grenade in grenades:
            self._grenades[grenade.name] = grenade

        for item in items:
            self._items[item.name] = item

        for key in keys:
            self._keys[key.name] = key

    async def fetch_ammunition(self, query: str = None) -> List[Ammunition]:
        data = await self.__requester.get_ammunition(query)
        return [Ammunition(d) for d in data]

    async def fetch_armor(self, query: str = None) -> List[Armor]:
        data = await self.__requester.get_armor(query)
        return [Armor(d) for d in data]

    async def fetch_backpack(self, query: str = None) -> List[Backpack]:
        data = await self.__requester.get_backpack(query)
        return [Backpack(d) for d in data]

    async def fetch_barter(self, query: str = None) -> List[Barter]:
        data = await self.__requester.get_barter(query)
        return [Barter(d) for d in data]

    async def fetch_food(self, query: str = None) -> List[Food]:
        data = await self.__requester.get_food(query)
        return [Food(d) for d in data]

    async def fetch_grenade(self, query: str = None) -> List[Grenade]:
        data = await self.__requester.get_grenade(query)
        return [Grenade(d) for d in data]

    async def fetch_headphone(self, query: str = None) -> List[Headphone]:
        data = await self.__requester.get_headphone(query)
        return [Headphone(d) for d in data]

    async def fetch_item(self, query: str = None) -> List[Item]:
        data = await self.__requester.get_item(query)
        return [Item(d) for d in data]

    async def fetch_key(self, query: str = None) -> List[Key]:
        data = await self.__requester.get_key(query)
        return [Key(d) for d in data]

    async def fetch_map(self, query: str = None) -> List[Map]:
        data = await self.__requester.get_map(query)
        return [Map(d) for d in data]

    async def __aenter__(self) -> TVCClient:
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.__requester._session.close()

    @property
    def ammunition(self) -> List[Ammunition]:
        return list(self._ammunition.values())

    @property
    def armors(self) -> List[Armor]:
        return list(self._armors.values())

    @property
    def backpacks(self) -> List[Backpack]:
        return list(self._backpacks.values())

    @property
    def barters(self) -> List[Barter]:
        return list(self._barters.values())

    @property
    def foods(self) -> List[Food]:
        return list(self._foods.values())

    @property
    def grenades(self) -> List[Grenade]:
        return list(self._grenades.values())

    @property
    def items(self) -> List[Item]:
        return list(self._items.values())

    @property
    def keys(self) -> List[Key]:
        return list(self._keys.values())



from typing import Optional, TypedDict

from .cell import Cell
from .item import ItemBase

_Key = TypedDict(
    '_Key',
    {
        'Fixed Price': Optional[int],
        'Unlootable': str,
        'Discarding Block': str,
        'MaximumNumber Of Usage': str,
    },
    total=False,
)


class Key(_Key, ItemBase, Cell):
    pass



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Headphone = TypedDict(
    '_HeadPhone',
    {
        'Blocks Earpiece': str,
        'Blocks Eyewear': str,
        'Blocks Headwear': str,
        'Blocks Face Cover': str,
        'Distortion': str,
        'Compressor Treshold': str,
        'Compressor Attack': str,
        'Compressor Release': str,
        'Compressor Gain': str,
        'Cutoff Frequency': str,
        'Resonance': str,
        'Compressor Volume': str,
        'Ambient Volume': str,
        'Dry Volume': str,
        'Can be sold on flea market': str,
    }
)


class Headphone(_Headphone, ItemBase, Cell):
    pass



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.headphone import Headphone as HeadphonePayload

__all__ = ('Headphone',)


class Headphone:

    def __init__(self, payload: HeadphonePayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __eq__(self, other: Headphone) -> bool:
        return isinstance(other, Headphone) and self.id == other.id

    def __ne__(self, other: Headphone) -> bool:
        return not self.__eq__(other)

    def _update(self, data: HeadphonePayload) -> None:
        self.block_earpiece: bool = 'true' == data['Blocks Earpiece']
        self.block_eyewear: bool = 'true' == data['Blocks Eyewear']
        self.block_headwear: bool = 'true' == data['Blocks Headwear']
        self.block_face_cover: bool = 'true' == data['Blocks Face Cover']
        self.distortion: float = float(data['Distortion'])

        self.compressor_treshold: int = int(data['Compressor Treshold'])
        self.compressor_attack: int = int(data['Compressor Treshold'])
        self.compressor_release: int = int(data['Compressor Release'])
        self.compressor_gain: int = int(data['Compressor Gain'])

        self.cutoff_ferquency: int = int(data['Cutoff Frequency'])
        self.resonance: int = int(data['Resonance'])

        self.compressor_volume: int = int(data['Compressor Volume'])
        self.ambient_volume: int = int(data['Ambient Volume'])
        self.dry_volume: int = int(data['Dry Volume'])

        self.cell_height: int = int(data['Cell Height'])
        self.width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.banned_on_flea: bool = data['Can be sold on flea market'] == 'false'
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])



from __future__ import annotations

from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from .types.food import Food as FoodPayload

__all__ = ('Food',)


class Food:

    def __init__(self, payload: FoodPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __eq__(self, other: Food) -> bool:
        return isinstance(other, Food) and self.id == other.id

    def __ne__(self, other: Food) -> bool:
        return not self.__eq__(other)

    def _update(self, data: FoodPayload) -> None:
        self.use_time: int = int(data['Use Time'])
        self.effect_type: str = data['Effect Type']
        self.max_resource: int = int(data['Max Resource'])
        self.stimulator_buffs: str = data['Stimulator Buffs']
        self.health_effects: List[str] = list(data[' Health Effects'])
        self.removes_effects: List[str] = list(data['Removes Effects'])

        self.cell_height: int = int(data['Cell Height'])
        self.width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])



from __future__ import annotations

from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from .types.armor import Armor as ArmorPayload

__all__ = ('Armor',)


class Armor:

    def __init__(self, payload: ArmorPayload) -> None:
        self.name: str = payload['Name']
        self.item_id: str = payload['Item Weight']
        self.armor_class: int = int(payload['Armor Class'])
        self.materials: str = payload['Materials']
        self.protection_zones: List[str] = list(payload['Protection Zones'])

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __eq__(self, other: Armor) -> bool:
        return isinstance(other, Armor) and self.item_id == other.item_id

    def __ne__(self, other: Armor) -> bool:
        return not self.__eq__(other)

    def _update(self, data: ArmorPayload) -> None:
        self.max_durability: int(data['Max Durability'])
        self.effective_durability: float = data['Effective Durability']
        self.movement_speed_penalty: int = int(data['Movement Speed Penalty'])
        self.turn_speed_penalty: int = int(data['Turn Speed Penalty'])
        self.blunt_throughput: float = float(data['Blunt Throughput'])
        self.repair_cost: int = int(data['Repair Cost'])
        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.banned_on_flea: bool = data['Can be sold on flea market'] == 'false'
        self.discard_limit: int = int(data['Discard Limit'])



from typing import TypedDict

Cell = TypedDict(
    'Cell',
    {
        'Cell Height': str,
        'Cell Width': str,
        'Item Weight': str,
        'Max Stack Size': str,
        'Discard Limit': str,
    }
)



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Armor = TypedDict(
    '_Armor',
    {
        'Armor Class': str,
        'Materials': str,
        'Protection Zones': str,
        'Max Durability': str,
        'Effective Durability': float,
        'Movement Speed Penalty': str,
        'Turn Speed Penalty': str,
        'Ergonomics Penalty': str,
        'Blunt Throughput': str,
        'Repair Cost': str,
        'Can be sold on flea market': str,
    },
    total=False,
)


class Armor(_Armor, ItemBase, Cell):
    pass



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.grenade import Grenade as GrenadePayload

__all__ = ('Grenade',)


class Grenade:

    def __init__(self, payload: GrenadePayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def _update(self, data: GrenadePayload) -> None:
        self.can_hide_during_throw: bool = 'true' == data['Can Be Hidden During Throw']
        self.contusion_distance: int = int(data['Contusion Distance'])
        self.explosion_delay: float = float(data['Explosion Delay'])
        self.fragments_count: int = int(data['Fragments Count'])
        self.max_explosion_distance: int = int(data['Max Explosion Distance'])
        self.min_explosion_distance: int = int(data['Min Explosion Distance'])
        self.strength: int = int(data['Strength'])
        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.banned_on_flea: bool = data['Can be sold on flea market'] == 'false'
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.maps import Map as MapPayload

__all__ = ('Map',)


class Map:

    def __init__(self, payload: MapPayload) -> None:
        self.name: str = payload['Name']
        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __int__(self) -> int:
        return self.raid_timer

    def __bool__(self) -> bool:
        return self.enabled

    def __eq__(self, other: Map) -> bool:
        return isinstance(other, Map) and self.internal_name == other.internal_name

    def __ne__(self, other: Map) -> bool:
        return not self.__eq__(other)

    def _update(self, data: MapPayload) -> None:
        self.enabled: bool = data['Map Enabled']
        self.locked: bool = data['Map Locked']
        self.internal_name: str = data['Map Internal Name']
        self.avg_player_level: int = int(data['Avg. Player Level'])
        self.raid_timer: int = data['Raid Timer']
        self.max_players: int = int(data['Max Players'])
        self.min_players: int = int(data['Min Players'])
        self.required_player_level: int = int(data['Required Player Level'])



"""
Tarkov Changes API
~~~~~~~~~~~~~~~~~~

A wrapper for the Tarkov Changes API.

:copyright: (c) 2022-present Hostagen
:license: MIT, see LICENSE for more details.

"""

__version__ = 'v0.2.4'

from .ammunition import *
from .armor import *
from .backpack import *
from .barter import *
from .client import *
from .enums import *
from .food import *
from .grenade import *
from .headphone import *
from .item import *
from .key import *
from .maps import *
from . import utils



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Grenade = TypedDict(
    '_Grenade',
    {
        'Can Be Hidden During Throw': str,
        'Contusion Distance': str,
        'Explosion Delay': str,
        'Fragments Count': str,
        'Max Explosion Distance': str,
        'Min Explosion Distance': str,
        'Strength': str,
        'Can be sold on flea market': str,
    },
    total=False,
)


class Grenade(_Grenade, ItemBase, Cell):
    pass



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Food = TypedDict(
    '_Food',
    {
        'Use Time': str,
        'Effect Type': str,
        'Max Resource': str,
        'Stimulator Buffs': str,
        ' Health Effects': str,
        'Removes Effects': str,
    },
    total=False,
)


class Food(_Food, ItemBase, Cell):
    pass



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Backpack = TypedDict(
    '_Backpack',
    {
        'Blocks Armored Vest': str,
        'Speed Penalty (%)': str,
        'Can be sold on flea market': str,
    },
    total=False,
)


class Backpack(_Backpack, ItemBase, Cell):
    pass



from __future__ import annotations

from typing import Any, Dict, Union, Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from aiohttp import ClientResponse

    try:
        from requester import Response

        _ResponseType = Union[ClientResponse, Response]

    except ModuleNotFoundError:
        _ResponseType = ClientResponse

__all__ = (
    'TVCException',
    'RequesterException',
    'InvalidRequest',
    'NotFound',
)


class TVCException(Exception):
    pass


class RequesterException(TVCException):

    def __init__(
        self,
        response: _ResponseType,
        message: Optional[Union[str, Dict[str, Any]]]
    ) -> None:
        self.response: _ResponseType = response
        self.status: int = response.status

        self.text: str

        if isinstance(message, dict):
            self.text = message.get('msg')

        else:
            self.text = message or ""

        fmt = "{0.status} {0.reason} (error code: {1})"

        if len(self.text):
            fmt += ": {2}"

        super().__init__(fmt.format(self.response, self.status, self.text))


class InvalidRequest(RequesterException):
    pass


class NotFound(RequesterException):
    pass



from __future__ import annotations

from typing import Union, TYPE_CHECKING

if TYPE_CHECKING:
    from .types.key import Key as KeyPayload

__all__ = ('Key',)


class Key:

    def __init__(self, payload: KeyPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __int__(self) -> int:
        return self.maximum_usage

    def __eq__(self, other: Key) -> bool:
        return isinstance(other, Key) and self.id == other.id

    def __ne__(self, other: Key) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        attrs = (
            ('id', self.id),
            ('name', self.name),
            ('fixed_price', self.fixed_price),
            ('maximum_usage', self.maximum_usage)
        )

        joined = ' '.join('%s=%r' % t for t in attrs)
        return f'<{self.__class__.__name__} {joined}>'

    def _update(self, data: KeyPayload) -> None:
        fixed_price: Union[int, None] = data['Fixed Price']

        if fixed_price is None:
            fixed_price = 0

        self.fixed_price = fixed_price
        self.unlootable: bool = 'true' == data['Unlootable']
        self.discarding_block: bool = 'true' == data['Discarding Block']
        self.maximum_usage: int = int(data['MaximumNumber Of Usage'] or 0)
        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])



from __future__ import annotations

from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from .types.item import Item as ItemPayload

__all__ = ('Item',)


class Item:

    def __init__(self, payload: ItemPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __eq__(self, other: Item) -> bool:
        return isinstance(other, Item) and self.id == other.id

    def __ne__(self, other: Item) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        attrs = (
            ('id', self.id),
            ('name', self.name),
        )

        joined = ' '.join('%s=%r' % t for t in attrs)
        return f'<{self.__class__.__name__} {joined}>'

    def _update(self, data: ItemPayload) -> None:
        self.props: List[str] = list(data['props'])



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Barter = TypedDict(
    '_Barter',
    {
        'Examine EXP': str,
        'Quest Only Item': str,
    },
    total=False,
)


class Barter(_Barter, ItemBase, Cell):
    pass



import types
from collections import namedtuple
from typing import (
    TYPE_CHECKING,
    Any,
    ClassVar,
    Dict,
    List,
)

__all__ = (
    'CaliberType',
)


def _create_value_cls(name, comparable):
    cls = namedtuple(f"_EnumValue_{name}", "name value")
    cls.__repr__ = lambda self: f"<{name}.{self.name}: {self.value!r}>"
    cls.__str__ = lambda self: f"{name}.{self.name}"
    if comparable:
        cls.__le__ = lambda self, other: isinstance(other, self.__class__) and self.value <= other.value
        cls.__ge__ = lambda self, other: isinstance(other, self.__class__) and self.value >= other.value
        cls.__lt__ = lambda self, other: isinstance(other, self.__class__) and self.value < other.value
        cls.__gt__ = lambda self, other: isinstance(other, self.__class__) and self.value > other.value
    return cls


def _is_descriptor(obj):
    return hasattr(obj, "__get__") or hasattr(obj, "__set__") or hasattr(obj, "__delete__")


class EnumMeta(type):
    if TYPE_CHECKING:
        __name__: ClassVar[str]
        _enum_member_names_: ClassVar[List[str]]
        _enum_member_map_: ClassVar[Dict[str, Any]]
        _enum_value_map_: ClassVar[Dict[Any, Any]]

    def __new__(cls, name, bases, attrs, *, comparable: bool = False):
        value_mapping = {}
        member_mapping = {}
        member_names = []

        value_cls = _create_value_cls(name, comparable)
        for key, value in list(attrs.items()):
            is_descriptor = _is_descriptor(value)
            if key[0] == "_" and not is_descriptor:
                continue

            # Special case classmethod to just pass through
            if isinstance(value, classmethod):
                continue

            if is_descriptor:
                setattr(value_cls, key, value)
                del attrs[key]
                continue

            try:
                new_value = value_mapping[value]
            except KeyError:
                new_value = value_cls(name=key, value=value)
                value_mapping[value] = new_value
                member_names.append(key)

            member_mapping[key] = new_value
            attrs[key] = new_value

        attrs["_enum_value_map_"] = value_mapping
        attrs["_enum_member_map_"] = member_mapping
        attrs["_enum_member_names_"] = member_names
        attrs["_enum_value_cls_"] = value_cls
        actual_cls = super().__new__(cls, name, bases, attrs)
        value_cls._actual_enum_cls_ = actual_cls  # type: ignore
        return actual_cls

    def __iter__(cls):
        return (cls._enum_member_map_[name] for name in cls._enum_member_names_)

    def __reversed__(cls):
        return (cls._enum_member_map_[name] for name in reversed(cls._enum_member_names_))

    def __len__(cls):
        return len(cls._enum_member_names_)

    def __repr__(cls):
        return f"<enum {cls.__name__}>"

    @property
    def __members__(cls):
        return types.MappingProxyType(cls._enum_member_map_)

    def __call__(cls, value):
        try:
            return cls._enum_value_map_[value]
        except (KeyError, TypeError):
            raise ValueError(f"{value!r} is not a valid {cls.__name__}")

    def __getitem__(cls, key):
        return cls._enum_member_map_[key]

    def __setattr__(cls, name, value):
        raise TypeError("Enums are immutable.")

    def __delattr__(cls, attr):
        raise TypeError("Enums are immutable")

    def __instancecheck__(self, instance):
        # isinstance(x, Y)
        # -> __instancecheck__(Y, x)
        try:
            return instance._actual_enum_cls_ is self
        except AttributeError:
            return False


if TYPE_CHECKING:
    from enum import Enum
else:

    class Enum(metaclass=EnumMeta):
        @classmethod
        def try_value(cls, value):
            try:
                return cls._enum_value_map_[value]
            except (KeyError, TypeError):
                return value


class EnumMeta(type):
    if TYPE_CHECKING:
        __name__: ClassVar[str]
        _enum_member_names_: ClassVar[List[str]]
        _enum_member_map_: ClassVar[Dict[str, Any]]
        _enum_value_map_: ClassVar[Dict[Any, Any]]

    def __new__(cls, name, bases, attrs, *, comparable: bool = False):
        value_mapping = {}
        member_mapping = {}
        member_names = []

        value_cls = _create_value_cls(name, comparable)
        for key, value in list(attrs.items()):
            is_descriptor = _is_descriptor(value)
            if key[0] == "_" and not is_descriptor:
                continue

            # Special case classmethod to just pass through
            if isinstance(value, classmethod):
                continue

            if is_descriptor:
                setattr(value_cls, key, value)
                del attrs[key]
                continue

            try:
                new_value = value_mapping[value]
            except KeyError:
                new_value = value_cls(name=key, value=value)
                value_mapping[value] = new_value
                member_names.append(key)

            member_mapping[key] = new_value
            attrs[key] = new_value

        attrs["_enum_value_map_"] = value_mapping
        attrs["_enum_member_map_"] = member_mapping
        attrs["_enum_member_names_"] = member_names
        attrs["_enum_value_cls_"] = value_cls
        actual_cls = super().__new__(cls, name, bases, attrs)
        value_cls._actual_enum_cls_ = actual_cls  # type: ignore
        return actual_cls

    def __iter__(cls):
        return (cls._enum_member_map_[name] for name in cls._enum_member_names_)

    def __reversed__(cls):
        return (cls._enum_member_map_[name] for name in reversed(cls._enum_member_names_))

    def __len__(cls):
        return len(cls._enum_member_names_)

    def __repr__(cls):
        return f"<enum {cls.__name__}>"

    @property
    def __members__(cls):
        return types.MappingProxyType(cls._enum_member_map_)

    def __call__(cls, value):
        try:
            return cls._enum_value_map_[value]
        except (KeyError, TypeError):
            raise ValueError(f"{value!r} is not a valid {cls.__name__}")

    def __getitem__(cls, key):
        return cls._enum_member_map_[key]

    def __setattr__(cls, name, value):
        raise TypeError("Enums are immutable.")

    def __delattr__(cls, attr):
        raise TypeError("Enums are immutable")

    def __instancecheck__(self, instance):
        # isinstance(x, Y)
        # -> __instancecheck__(Y, x)
        try:
            return instance._actual_enum_cls_ is self
        except AttributeError:
            return False


if TYPE_CHECKING:
    from enum import Enum

else:

    class Enum(metaclass=EnumMeta):
        @classmethod
        def try_value(cls, value):
            try:
                return cls._enum_value_map_[value]
            except (KeyError, TypeError):
                return value


class CaliberType(Enum):
    Caliber12g = '12x70mm'
    Caliber127x108 = '12.7x108mm'
    Caliber127x55 = '12.7x55mm STs-130'
    Caliber20g = '20x70mm'
    Caliber57x28 = '5.7x28mm FN'
    Caliber1143x23ACP = '.45 ACP'
    Caliber9x21 = '9x21mm Gyurza'
    Caliber9x19PARA = '9x19mm Parabellum'
    Caliber86x70 = '.338 Lapua Magnum'
    Caliber762x35 = '.300 Blackout'
    Caliber762x54R = '7.62x54mmR'
    Caliber23x75 = '23x75mm'
    Caliber46x30 = '4.6x30mm HK'
    Caliber40x46 = '40x46mm'
    Caliber26x75 = '26x75mm'
    Caliber762x51 = '7.62x51mm NATO'
    Caliber762x25TT = '7.62x25mm Tokarev'
    Caliber762x39 = '7.62x39mm'
    Caliber9x18PM = '9x18mm Makarov'
    Caliber556x45NATO = '5.56x45mm NATO'
    Caliber366TKM = '.366 TKM'
    Caliber9x33R = '.357 Magnum'
    Caliber30x29 = '30x29mm'
    Caliber40mmRU = '40mm VOG-25'
    Caliber545x39 = '5.45x39mm'
    Caliber9x39 = '9x39mm'

    def __str__(self):
        return self.name



from __future__ import annotations

import json
import aiohttp
import asyncio

from typing import (
    Any,
    ClassVar,
    Coroutine,
    Optional,
    Union,
    Dict,
    List,
    TypeVar,
    TYPE_CHECKING,
)

from urllib.parse import quote as _uriquote

from . import utils
from .exceptions import InvalidRequest, NotFound

if TYPE_CHECKING:
    from .types.armor import Armor as ArmorPayload
    from .types.ammunition import Ammunition as AmmunitionPayload
    from .types.backpack import Backpack as BackpackPayload
    from .types.barter import Barter as BarterPayload
    from .types.food import Food as FoodPayload
    from .types.grenade import Grenade as GrenadePayload
    from .types.headphone import Headphone as HeadphonePayload
    from .types.item import Item as ItemPayload
    from .types.key import Key as KeyPayload
    from .types.maps import Map as MapPayload

    T = TypeVar('T')
    BE = TypeVar('BE', bound=BaseException)
    MU = TypeVar('MU', bound='MaybeUnlock')
    Response = Coroutine[Any, Any, T]


async def json_or_text(response: aiohttp.ClientResponse) -> Union[Dict[str, Any], str]:
    text = await response.text(encoding='utf-8')

    try:

        if response.headers['content-type'] == 'application/json':
            return json.loads(text)

    except KeyError:
        pass

    return text


class Route:
    BASE: ClassVar[str] = 'https://api.tarkov-changes.com/v1'

    def __init__(self, method: str, path: str, **parameters: Any):
        self.path: str = path
        self.method: str = method

        url = "{}{}".format(self.BASE, path)

        if parameters:
            url = url.format_map({k: _uriquote(v) if isinstance(v, str) else v for k, v in parameters.items()})

        self.url = url


class HTTPRequester:

    def __init__(
        self,
        *,
        token: str,
        session: aiohttp.ClientSession = aiohttp.ClientSession(),
        loop: Optional[asyncio.AbstractEventLoop] = None,
    ) -> None:
        self._loop: asyncio.AbstractEventLoop = asyncio.get_event_loop() if loop is None else loop
        self._session = session
        self.token: str = token

    async def request(self, route: Route, **kwargs: Any) -> Any:
        method = route.method
        url = route.url

        headers: Dict[str, str] = {
            "AUTH-TOKEN": self.token,
        }

        if 'json' in kwargs:
            headers['Content-Type'] = 'application/json'
            kwargs['data'] = utils._to_json(kwargs.pop('json'))

        kwargs['headers'] = headers

        async with self._session.request(method, url, **kwargs) as response:
            data = await json_or_text(response)

            if response.status == 204:
                return []

            if 300 > response.status >= 200:
                return data['results']

            if response.status == 401:
                raise InvalidRequest(response, data)

            if response.status == 404:
                raise NotFound(response, data)

    async def close(self) -> None:
        await self._session.close()

    def get_ammunition(self, query: str = None) -> Response[List[AmmunitionPayload]]:
        url = '/ammo'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_armor(self, query: str = None) -> Response[List[ArmorPayload]]:
        url = '/armor'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_backpack(self, query: str = None) -> Response[List[BackpackPayload]]:
        url = '/backpacks'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_barter(self, query: str = None) -> Response[List[BarterPayload]]:
        url = '/barters'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_food(self, query: str = None) -> Response[List[FoodPayload]]:
        url = '/food'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_grenade(self, query: str = None) -> Response[List[GrenadePayload]]:
        url = '/grenades'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_headphone(self, query: str = None) -> Response[List[HeadphonePayload]]:
        url = '/headphones'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_item(self, query: str = None) -> Response[List[ItemPayload]]:
        url = '/items'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_key(self, query: str = None) -> Response[List[KeyPayload]]:
        url = '/keys'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)

    def get_map(self, query: str = None) -> Response[List[MapPayload]]:
        url = '/maps'

        if query:
            url += f'?query={query}'

        r = Route('GET', url)
        return self.request(r)



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.barter import Barter as BarterPayload

__all__ = ('Barter',)


class Barter:

    def __init__(self, payload: BarterPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']

        self._update(payload)

    def _update(self, data: BarterPayload) -> None:
        self.examine_exp: int = int(data['Examine EXP'])
        self.quest_only_item: bool = 'true' == data['Quest Only Item']

        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])



from typing import TypedDict

from .cell import Cell
from .item import ItemBase

_Ammunition = TypedDict(
    '_Ammunition',
    {
        'Caliber': str,
        'Accuracy': str,
        'Recoil': str,
        'Flesh Damage': str,
        'Penetration Power': str,
        'Armor Damage': str,
        'Frag Chance': str,
        'Durability Burn': str,
        'Stamina Burn per Dmg': str,
        'Projectile Speed': str,
        'Misfire Chance': str,
        'Penetration Chance': str,
        'Ricochet Chance': str,
        'Heavy Bleeding Delta': str,
        'Light Bleeding Delta': str,
        'Ballistic Coefficient': str,
    },
    total=False,
)


class Ammunition(_Ammunition, ItemBase, Cell):
    pass



import re

from setuptools import setup

requirements = []

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

version = ''
with open('tvc/__init__.py') as f:
    search = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE)

    if search is not None:
        version = search.group(1)

    else:
        raise RuntimeError("Could not grab version string")

if not version:
    raise RuntimeError("version is not set")

readme = ''

with open('README.rst') as f:
    readme = f.read()

packages = [
    'tvc',
    'tvc.types'
]

setup(
    name='tarkov-changes.py',
    url='https://github.com/Hostagen/tarkov-changes.py',
    description='async API wrapper for Tarkov Changes written in Python.',
    long_description=readme,
    long_description_content_type="text/x-rst",
    version=version,
    packages=packages,
    author='Hostagen',
    license='MIT',
    python_requires='>=3.8.0',
)



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.backpack import Backpack as BackPackPayload

__all__ = ('Backpack',)


class Backpack:

    def __init__(self, payload: BackPackPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']
        self.blocks_armor_vest: bool = 'true' == payload['Blocks Armored Vest']

        self._update(payload)

    def _update(self, data: BackPackPayload) -> None:
        self.speed_penalty_percent: int = int(data['Speed Penalty (%)'])
        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])

        self.weight: float = float(data['Item Weight'])
        self.banned_on_flea: bool = data['Can be sold on flea market'] == 'false'
        self.discard_limit: int = int(data['Discard Limit'])
        self.max_stack_size: int = int(data['Max Stack Size'])



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .types.ammunition import Ammunition as AmmunitionPayload

__all__ = ('Ammunition',)


class Ammunition:

    def __init__(self, payload: AmmunitionPayload) -> None:
        self.name: str = payload['Name']
        self.id: str = payload['Item ID']
        self.caliber: str = payload['Caliber']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __int__(self) -> int:
        return self.flesh_damage

    def __eq__(self, other: Ammunition) -> bool:
        return isinstance(other, Ammunition) and self.id == other.id

    def __ne__(self, other: Ammunition) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        attrs = (
            ('id', self.id),
            ('name', self.name),
            ('flesh_damage', self.flesh_damage),
            ('caliber', self.caliber),
        )

        joined = ' '.join('%s=%r' % t for t in attrs)
        return f'<{self.__class__.__name__} {joined}>'

    def _update(self, data: AmmunitionPayload):
        self.flesh_damage: int = int(data['Flesh Damage'])
        self.penetration_power: int = int(data['Penetration Power'])
        self.armor_damage: int = int(data['Armor Damage'])
        self.accuracy: int = int(data['Accuracy'])
        self.recoil: int = int(data['Recoil'])
        self.frag_chance: float = float(data['Frag Chance'])
        self.durability_burn: float = float(data['Durability Burn'])
        self.stamina_burn_per_damage: float = float(data['Stamina Burn per Dmg'])
        self.projectile_speed: int = int(data['Projectile Speed'])
        self.misfire_chance: float = float(data['Misfire Chance'])
        self.penetration_chance: float = float(data['Penetration Chance'])
        self.heavy_bleeding_delta: float = float(data['Heavy Bleeding Delta'])
        self.light_bleeding_delta: float = float(data['Light Bleeding Delta'])
        self.ballistic_coefficient: float = float(data['Ballistic Coefficient'])
        self.cell_height: int = int(data['Cell Height'])
        self.cell_width: int = int(data['Cell Width'])
        self.weight: float = float(data['Item Weight'])
        self.max_stack_size: int = int(data['Max Stack Size'])
        self.discard_limit: int = int(data['Discard Limit'])
